Public Class Feiertagsberechnung
    Public Function Ostersonntag(ByVal Jahr As Integer) As Date
        'Osterfunktion nach Carl Friedrich Gau (1800). Rckgabewert
        ' ist das Datum des Ostersonntags im angegebenen (ersatzweise:
        ' aktuellen) Jahr. Gltigkeitsbereich: 1583 - 8702 (auf das
        ' Auslsen von Laufzeitfehlern bei Unter- oder berschreitung
        ' dieses Gltigkeitsbereichs wird hier verzichtet)
        Dim a, b, c, d, e, f As Integer


        ' Die "magische" Gauss-Formel anwenden:
        a = Jahr Mod 19
        b = Jahr \ 100
        c = (8 * b + 13) \ 25 - 2
        d = b - (Jahr \ 400) - 2
        e = (19 * (Jahr Mod 19) + ((15 - c + d) Mod 30)) Mod 30
        If e = 28 Then
            If a > 10 Then
                e = 27
            End If
        ElseIf e = 29 Then
            e = 28
        End If
        f = (d + 6 * e + 2 * (Jahr Mod 4) + 4 * (Jahr Mod 7) + 6) Mod 7

        ' Rckgabewert als Datum bereitstellen
        Dim ZDatum As New Date(Jahr, 3, 1)
        Return ZDatum.AddDays(e + f + 21)
    End Function


    Public Function Feiertagsdatum(ByVal Feiertag As Feiertage, ByVal Jahr As Integer) As Date
        ' Gibt das Datum eines datumsfesten oder beweglichen Feiertags zurck.

        ' Feiertage ermitteln:
        Select Case Feiertag
            Case Feiertage.Neujahr
                Return New Date(Jahr, 1, 1)

            Case Feiertage.Erscheinungsfest
                Return New Date(Jahr, 1, 6)

            Case Feiertage.Karfreitag
                Return Ostersonntag(Jahr).AddDays(-2)

            Case Feiertage.Ostersonntag
                Return Ostersonntag(Jahr)

            Case Feiertage.Ostermontag
                Return Ostersonntag(Jahr).AddDays(1)

            Case Feiertage.Maifeiertag
                Return New Date(Jahr, 5, 1)

            Case Feiertage.ChristiHimmelfahrt
                Return Ostersonntag(Jahr).AddDays(39)

            Case Feiertage.Pfingstmontag
                Return Ostersonntag(Jahr).AddDays(50)

            Case Feiertage.Fronleichnam
                Return Ostersonntag(Jahr).AddDays(60)

            Case Feiertage.MariaeHimmelfahrt
                Return New Date(Jahr, 8, 15)

            Case Feiertage.TagDerEinheit
                Return New Date(Jahr, 10, 3)

            Case Feiertage.Reformationstag
                Return New Date(Jahr, 10, 31)

            Case Feiertage.Allerheiligen
                Return New Date(Jahr, 11, 1)

            Case Feiertage.BussUndBettag
                'Der Buss- und Bettag ist der Mittwoch vor dem Sonntag vor dem 1.Advent

                Dim VierterAdvent As DateTime
                Dim Weihnachten As New Date(Jahr, 12, 24)

                VierterAdvent = Weihnachten.AddDays(-Weihnachten.DayOfWeek)
                Return VierterAdvent.AddDays(-32)

            Case Feiertage.Weihnachtsfeiertag1
                Return New Date(Jahr, 12, 25)
            Case Feiertage.Weihnachtsfeiertag2
                Return New Date(Jahr, 12, 26)
        End Select

    End Function


    Public Function Bundeslandname(ByVal Bundesland As Bundeslaender) As String
        ' Service-Funktion fr Ausgabezwecke: Liefert zu einem Wert der Aufzhlung
        ' "Bundeslaender" den Klartextnamen des durch den Wert reprsentierten
        ' Bundeslandes zurck.

        Select Case Bundesland
            Case Bundeslaender.BadenWuerttemberg : Return "Baden-Wrttemberg"
            Case Bundeslaender.Bayern : Return "Bayern"
            Case Bundeslaender.Berlin : Return "Berlin"
            Case Bundeslaender.Brandenburg : Return "Brandenburg"
            Case Bundeslaender.Bremen : Return "Bremen"
            Case Bundeslaender.Hamburg : Return "Hamburg"
            Case Bundeslaender.Hessen : Return "Hessen"
            Case Bundeslaender.MecklenburgVorpommern : Return "Mecklenburg-Vorpommern"
            Case Bundeslaender.Niedersachsen : Return "Niedersachsen"
            Case Bundeslaender.NordrheinWestfalen : Return "Nordrhein-Westfalen"
            Case Bundeslaender.RheinlandPfalz : Return "Rheinland-Pfalz"
            Case Bundeslaender.Saarland : Return "Saarland"
            Case Bundeslaender.Sachsen : Return "Sachsen"
            Case Bundeslaender.SachsenAnhalt : Return "Sachsen-Anhalt"
            Case Bundeslaender.SchleswigHolstein : Return "Schleswig-Holstein"
            Case Bundeslaender.Thueringen : Return "Thringen"
        End Select
        Return Nothing
    End Function
    Public Function Feiertagsname(ByVal Feiertag As Feiertage) As String
        ' Service-Funktion fr Ausgabezwecke: Liefert zu einem Wert der Aufzhlung
        ' "Feiertage" den Klartextnamen des durch den Wert reprsentierten
        ' Feiertages zurck.

        Select Case Feiertag
            Case Feiertage.Allerheiligen : Return "Allerheiligen"
            Case Feiertage.BussUndBettag : Return "Buss- und Bettag"
            Case Feiertage.ChristiHimmelfahrt : Return "Christi Himmelfahrt"
            Case Feiertage.Erscheinungsfest : Return "Erscheinungsfest"
            Case Feiertage.Fronleichnam : Return "Fronleichnam"
            Case Feiertage.Karfreitag : Return "Karfreitag"
            Case Feiertage.Maifeiertag : Return "1. Mai"
            Case Feiertage.MariaeHimmelfahrt : Return "Mariae Himmelfahrt"
            Case Feiertage.Neujahr : Return "Neujahr"
            Case Feiertage.Ostermontag : Return "Ostermontag"
            Case Feiertage.Ostersonntag : Return "Ostersonntag"
            Case Feiertage.Pfingstmontag : Return "Pfingstmontag"
            Case Feiertage.Reformationstag : Return "Reformationstag"
            Case Feiertage.TagDerEinheit : Return "Tag der Deutschen Einheit"
            Case Feiertage.Weihnachtsfeiertag1 : Return "1. Weihnachtsfeiertag"
            Case Feiertage.Weihnachtsfeiertag2 : Return "2. Weihnachtsfeiertag"
        End Select
        Return Nothing
    End Function

    Public Function IstFeiertagIn(ByVal Feiertag As Feiertage, _
    Optional ByVal Bundesland As Bundeslaender = 0) As Boolean

        ' Gibt zurck, ob der im Parameter Feiertag angegebene Feiertag in
        ' einem angegebenen Bundesland begangen wird (True) oder nicht (False).
        ' Wird fr den Parameter Bundesland kein Wert bergeben, wird zurckgegeben,
        ' ob der Feiertag ein bundesweit einheitlicher Feiertag ist.
        '
        ' HINWEIS: Feiertage, die nur in einzelnen Gemeinden eines Bundeslandes
        '          begangen werden (Fronleichnam: Sachsen, Thringen; Mari
        '          Himmelfahrt: Bayern), werden von dieser Funktion als in diesen
        '          Bundeslndern als NICHT begangen behandelt! Hinweise, wie Sie
        '          dieses Verhalten bei Bedarf ndern knnen, finden Sie in diesem
        '          Sourcecode.
        '
        ' TIPP:    Bei bergabe durch den Or- oder +-Operator kumulierter Werte
        '          fr den Parameter Bundesland knnen Sie feststellen, ob dieser
        '          Feiertag in mehreren Bundeslndern begangen wird. Beispiel:
        '          IstFeiertagInBayernUndInHessen = _
        '            IstFeiertagIn(ftFronleichnam, ftBayern + ftHessen)

        ' Bundesweit einheitliche Feiertage
        ' Wurde kein Wert fr Bundesland bergeben, wird die Funktion hier verlassen
        Select Case Feiertag
            Case Feiertage.Neujahr, Feiertage.Karfreitag, Feiertage.Ostersonntag, Feiertage.Ostermontag, _
                 Feiertage.Maifeiertag, Feiertage.ChristiHimmelfahrt, Feiertage.Pfingstmontag, _
                 Feiertage.TagDerEinheit, Feiertage.Weihnachtsfeiertag1, Feiertage.Weihnachtsfeiertag2
                Return True
        End Select


        ' Bundesweit nicht einheitliche Feiertage
        Select Case Feiertag
            Case Feiertage.Erscheinungsfest
                If Bundesland And (Bundeslaender.BadenWuerttemberg Or Bundeslaender.Bayern Or Bundeslaender.SachsenAnhalt) Then
                    Return True
                End If
            Case Feiertage.Fronleichnam
                ' HINWEIS: In Sachsen-Anhalt und Thringen wird Fronleichnam
                ' nur vereinzelt als Feiertag begangen. Fr diese Bundeslnder
                ' liefert diese Funktion generell FALSE zurck. ndern Sie
                ' dieses Verhalten bei Bedarf in diesem Select Case-Zweig.
                If Bundesland And (Bundeslaender.BadenWuerttemberg Or Bundeslaender.Bayern Or Bundeslaender.Hessen Or Bundeslaender.NordrheinWestfalen Or Bundeslaender.RheinlandPfalz Or Bundeslaender.Saarland) Then
                    Return True
                End If

            Case Feiertage.MariaeHimmelfahrt
                ' HINWEIS: In Bayern wird Mari Himmelfahrt nur in einzelnen
                ' Gemeinden als Feiertag begangen. Fr dieses Bundesland
                ' liefert diese Funktion generell FALSE zurck. ndern Sie
                ' dieses Verhalten bei Bedarf in diesem Select Case-Zweig.
                If Bundesland And Bundeslaender.Saarland Then
                    Return True
                End If

            Case Feiertage.Reformationstag
                If Bundesland And (Bundeslaender.Brandenburg Or Bundeslaender.MecklenburgVorpommern Or Bundeslaender.Sachsen Or Bundeslaender.SachsenAnhalt Or Bundeslaender.Thueringen) Then
                    Return True
                End If
            Case Feiertage.Allerheiligen
                If Bundesland And (Bundeslaender.BadenWuerttemberg Or Bundeslaender.Bayern Or Bundeslaender.NordrheinWestfalen Or Bundeslaender.RheinlandPfalz Or Bundeslaender.Saarland) Then
                    Return True
                End If
            Case Feiertage.BussUndBettag
                If Bundesland And Bundeslaender.Sachsen Then
                    Return True
                End If
        End Select
        Return False

    End Function





End Class
